use strict;
use vars qw(%RAD_REQUEST %RAD_REPLY %RAD_CHECK);
use Data::Dumper;

use constant	RLM_MODULE_REJECT=>    0;#  /* immediately reject the request */
use constant	RLM_MODULE_FAIL=>      1;#  /* module failed, don't reply */
use constant	RLM_MODULE_OK=>        2;#  /* the module is OK, continue */
use constant	RLM_MODULE_HANDLED=>   3;#  /* the module handled the request, so stop. */
use constant	RLM_MODULE_INVALID=>   4;#  /* the module considers the request invalid. */
use constant	RLM_MODULE_USERLOCK=>  5;#  /* reject the request (user is locked out) */
use constant	RLM_MODULE_NOTFOUND=>  6;#  /* user not found */
use constant	RLM_MODULE_NOOP=>      7;#  /* module succeeded without doing anything */
use constant	RLM_MODULE_UPDATED=>   8;#  /* OK (pairs modified) */
use constant	RLM_MODULE_NUMCODES=>  9;#  /* How many return codes there are */

# Same as src/include/radiusd.h
use constant	L_DBG=>   1;
use constant	L_AUTH=>  2;
use constant	L_INFO=>  3;
use constant	L_ERR=>   4;
use constant	L_PROXY=> 5;
use constant	L_ACCT=>  6;

# Function to handle authorize
sub authorize {
	return RLM_MODULE_OK;
}

# Function to handle authenticate
sub authenticate {
	return RLM_MODULE_OK;
}

# Function to handle preacct
sub preacct {
	return RLM_MODULE_OK;
}

# Function to handle accounting
sub accounting {
#	&radiusd::radlog(L_INFO, "accounting");
	return RLM_MODULE_OK;
}

# Function to handle accounting_start
sub accounting_start {
	#&radiusd::radlog(L_INFO, "accounting_start");
	&accounting_start_stop(1);
	return RLM_MODULE_OK;
}
# Function to handle accounting_stop
sub accounting_stop {
	#&radiusd::radlog(L_INFO, "accounting_stop");
	&accounting_start_stop(0);
	return RLM_MODULE_OK;
}

# Function to handle checksimul
sub checksimul {
	return RLM_MODULE_OK;
}

# Function to handle pre_proxy
sub pre_proxy {
	return RLM_MODULE_OK;
}

# Function to handle post_proxy
sub post_proxy {
	return RLM_MODULE_OK;
}

# Function to handle post_auth
sub post_auth {
	return RLM_MODULE_OK;
}

# Function to handle xlat
sub xlat {
	&radiusd::radlog(L_INFO, "xlat");
}

# Function to handle detach
sub detach {
	# Do some logging.
	&radiusd::radlog(L_INFO,"rlm_perl::Detaching. Reloading. Done.");
}

#
# Some functions that can be called from other functions
#

sub test_call {
	# Some code goes here
}

sub log_request_attributes {
	# This shouldn't be done in production environments!
	# This is only meant for debugging!
	for (keys %RAD_REQUEST) {
		&radiusd::radlog(L_INFO, "RAD_REQUEST: $_ = $RAD_REQUEST{$_}");
	}
}

sub accounting_start_stop ( $ ) {
	my $up = shift @_;
	my $if = $RAD_REQUEST{'NAS-Port-Id'}       if exists $RAD_REQUEST{'NAS-Port-Id'};
	my $ip = $RAD_REQUEST{'Framed-IP-Address'} if exists $RAD_REQUEST{'Framed-IP-Address'};
	my $ho = $RAD_REQUEST{'NAS-IP-Address'}    if exists $RAD_REQUEST{'NAS-IP-Address'};
	my $us = $RAD_REQUEST{'SQL-User-Name'}     if exists $RAD_REQUEST{'SQL-User-Name'};
	my $se = $RAD_REQUEST{'Acct-Session-Id'}   if exists $RAD_REQUEST{'Acct-Session-Id'};
	my $f6 = $RAD_REQUEST{'Framed-IPv6-Prefix'}    if exists $RAD_REQUEST{'Framed-IPv6-Prefix'};
	my $i6 = $RAD_REQUEST{'Framed-Interface-Id'}   if exists $RAD_REQUEST{'Framed-Interface-Id'};
	my $d6 = arr_to_str($RAD_REQUEST{'Delegated-IPv6-Prefix'}) if exists $RAD_REQUEST{'Delegated-IPv6-Prefix'};
	my $cl = "/usr/local/adminbcm/home/nuevocliente.php '$up' '$ip' '$if' '$ho' '$us' '$se' '$f6' '$i6' '$d6'";

	#&log_request_attributes();
	#&radiusd::radlog(L_INFO, "ini-$cl");
	if( -e "/usr/local/adminbcm/tmp/fnfwasinc" )
	{
		my $file="/tmp/adminbcmfw.txt";
		my $pipe="/tmp/adminbcmfw.pipe";
		if( -p $pipe )
		{
			$file=$pipe;
		}
		open(my $fd, "+>>$file");
		print $fd "/usr/bin/php -f $cl\n";
		close($fd);
	}
	else
	{
		system("/usr/local/adminbcm/clients/dhcp/clientedhcp.bin /usr/bin/php -f $cl");
	}
	#&radiusd::radlog(L_INFO, "fin-$cl");
}
sub arr_to_str {
	my $st = shift @_;
	if (ref($st) eq "ARRAY") {
		return join(',',@$st) ;
	}
	return $st;
}
